@@ -87,9 +87,9 @@ class uwDataAccess:
             where条件
         """
 
-        return recset(t, self.__session, w)
+        return recset[t](t, self.__session, w)
     
-    @loggerDecorator("Commit Seesion")    
+    @loggerDecorator("Commit Seesion")
     def commit(self):
     
         """ 
@@ -104,7 +104,7 @@ class uwDataAccess:
         # Session - Commit
         self.__session.commit()
     
-    @loggerDecorator("Commit Rollback")   
+    @loggerDecorator("Commit Rollback")
     def rollback(self):
 
         """ 
@@ -119,7 +119,7 @@ class uwDataAccess:
         # Session - Rollback
         self.__session.rollback()
         
-    @loggerDecorator("Close Seesion")    
+    @loggerDecorator("Close Seesion")
     def closeSession(self):
         
         """ 
@@ -215,6 +215,5 @@ class uwDataAccess:
             # SQLAlchemyErrorをThrow
             raise SQLAlchemyError
         
-        return updateCount    
-        
-             
\ No newline at end of file
+        return updateCount
+        
\ No newline at end of file
