@@ -1,17 +1,26 @@
 import inspect
+from pprint import pprint
 import pandas as pd
 import copy
+import LibHanger.Library.uwLogger as Logger
 from sqlalchemy.exc import SQLAlchemyError
 from sqlalchemy import and_
-import LibHanger.Library.uwLogger as Logger
+from typing import TypeVar, Generic
+from enum import Enum
+from LibHanger.Models.fields import fields
 
-class recset():
+T = TypeVar('T')
+
+class recset(Generic[T]):
     
     """
     レコードセットクラス
     """
 
-    class rowState():
+    __ROWSTATE_COLNAME__ = '__rowState'
+    """ 行状態列名 """
+
+    class rowState(Enum):
 
         """
         行状態クラス
@@ -56,16 +65,29 @@ class recset():
         # 主キー情報
         self.__primaryKeys = self.__getPrimaryKeys()
 
-        # レコードセット行初期化
-        self.rows = []
+        # 行情報初期化
+        self.__rows = []
+        self.__dfrows = None
         if self.__session is None or __where is None:
-            self.rows = []
+            self.__initRows()
         else:
             self.filter(__where)
         
-        # 現在行位置
+        # 現在行位置初期化
         self.__currentRowIndex = -1
+
+        # フィールドクラス
+        self.__fields = fields(self.__rows, self.__dfrows)
         
+    @property
+    def recSetName(self):
+        
+        """
+        レコードセット名
+        """
+        
+        return self.modelType.__tablename__
+    
     @property
     def session(self):
         
@@ -75,14 +97,98 @@ class recset():
         
         return self.__session
     
-    def setSession(self, __session):
-
+    @property
+    def rows(self):
+        
         """
-        DBセッションセット
+        行情報(List)
         """
+        
+        return self.__rows
+    
+    @property
+    def dfrows(self):
+        
+        """
+        行情報(DataFrame)
+        """
+        
+        return self.__fields.dfrows
+    
+    @property
+    def columns(self):
+        
+        """
+        カラム情報
+        """
+        
+        return self.__columns
+    
+    @property
+    def primaryKeys(self):
+        
+        """
+        主キー情報プロパティ
+        """
+        
+        return self.__primaryKeys
 
-        if self.__session is None:
-            self.__session = __session
+    @property
+    def State(self) ->int:
+        
+        """
+        行状態
+        """
+        
+        return self.__rows[self.__currentRowIndex].__rowState
+    
+    @State.setter
+    def State(self, __rowState):
+        
+        # rowsのカレント行の行状態をmodifiedに変更
+        if self.__rows[self.__currentRowIndex].__rowState == self.rowState.noChanged:
+            self.__rows[self.__currentRowIndex].__rowState = __rowState
+
+        # dfrowsのカレント行の行状態をmodifiedに変更
+        if self.__dfrows.loc[self.__currentRowIndex, self.__ROWSTATE_COLNAME__] == self.rowState.noChanged:
+            self.__dfrows.loc[self.__currentRowIndex, self.__ROWSTATE_COLNAME__] = __rowState
+
+    def fields(self, __columnName:T):
+        
+        """
+        レコードセットフィールド情報
+        """
+        
+        # カラム名をfieldsに渡す
+        self.__fields.columnName = __columnName
+        
+        # カレント行インデックスをfieldsに渡す
+        self.__fields.currentRowIndex = self.__currentRowIndex
+        
+        return self.__fields
+    
+    def __initdfrows(self):
+        
+        """
+        dfrows初期化(Dataframe)
+        """
+        
+        # 列リスト生成
+        colList = []
+        for col in self.__columns:
+            colList.append(col[0])
+        colList.append(self.__ROWSTATE_COLNAME__)
+        colList.sort()
+        
+        # DataFrame化
+        df = pd.DataFrame(columns=colList)
+        
+        # 主キーセット
+        if len(self.__primaryKeys) > 0:
+            df = df.set_index(self.__primaryKeys, drop=False)
+
+        # 戻り値を返す
+        return df
     
     def __getColumnAttr(self):
         
@@ -112,11 +218,11 @@ class recset():
         
         """
         
-        # 主キーリスト作成            
+        # 主キーリスト作成
         primaryKeys = []
         for col in self.__columns:
 
-            memberInvoke = getattr(self.modelType, col[0])            
+            memberInvoke = getattr(self.modelType, col[0])
             if memberInvoke.primary_key == True:
                 primaryKeys.append(col[0])
         
@@ -143,6 +249,113 @@ class recset():
         # 主キー条件リストをtupleに変換して返す
         return and_(*tuple(pKeyList))
     
+    def __rowSetting(self, row):
+        
+        """
+        行情報を生成する
+        
+        Parameters
+        ----------
+        row : any
+            行情報
+        """
+        
+        for col in self.__columns:
+
+            # Modelのインスタンス変数取得
+            memberInvoke = getattr(self.modelType, col[0])
+            # 既定値の設定
+            setattr(row, col[0], memberInvoke.default.arg)
+        
+        # 生成した行を返す
+        return row
+    
+    def __addRecrow(self):
+        
+        """
+        レコードセット行追加処理
+        """
+        
+        # カレント行インデックス++
+        self.__currentRowIndex += 1
+
+        # 行生成
+        row = self.modelType()
+        
+        # 行状態列を追加
+        row.__rowState = self.rowState.noChanged
+
+        # List - add
+        self.__rows.append(self.__rowSetting(row))
+
+        # DataFrame - add
+        self.__addRecrowDataFrame(row, self.__currentRowIndex)
+    
+        # 行状態をaddedに変更
+        self.State = self.rowState.added
+
+    def __addRecrowDataFrame(self, row, targetRowIndex):
+        
+        """
+        対象行をレコードソース(DataFrame)に追加する
+        """
+        
+        # 行情報コピー
+        lrow = []
+        lrow.append(row)
+        targetRow = self.__deepCopyRow(lrow)[0]
+
+        # deepcopyしたrowをdict化
+        dictRow = vars(targetRow)
+        dictRow_sorted = sorted(dictRow.items())
+        
+        # value取得
+        dfValList = [val[1] for val in dictRow_sorted]
+
+        # カレント行にvalue設定
+        self.__dfrows.loc[targetRowIndex] = dfValList
+    
+    def __editRecrow(self):
+        
+        """
+        レコードセットを編集状態にする
+        """
+        
+        # 行状態をmodifiedに変更
+        self.State = self.rowState.modified
+        
+    def __initRows(self):
+        
+        """
+        行情報をリセットする
+        """
+        
+        self.__rows = []
+        self.__dfrows = self.__initdfrows()
+    
+    def __deepCopyRow(self, rows:list) -> list:
+        
+        """
+        行情報をコピーする
+        """
+
+        # 行インスタンスをDeepCopy
+        targetRows = copy.deepcopy(rows)
+        # DataFrame化で不要な列を削除
+        for rowInstance in targetRows:
+            delattr(rowInstance, '_sa_instance_state')
+            
+        return targetRows
+
+    def setSession(self, __session):
+
+        """
+        DBセッションセット
+        """
+
+        if self.__session is None:
+            self.__session = __session
+
     def newRow(self):
         
         """
@@ -154,17 +367,13 @@ class recset():
 
         """
 
-        # 行生成
-        row = self.modelType()
-        # 行状態をaddedに変更
-        row.__rowState = self.rowState.added
-        # 行情報生成
-        return self.rowSetting(row)
+        # 新規行情報生成
+        self.__addRecrow()
     
     def editRow(self):
         
         """
-        編集行を取得する
+        レコードセットを編集状態にする
 
         Parameters
         ----------
@@ -172,61 +381,19 @@ class recset():
 
         """
         
-        # rowsのカレント行の行状態をmodifiedに変更
-        self.rows[self.__currentRowIndex].__rowState = self.rowState.modified
-        # rowsのカレント行を返す
-        return self.rows[self.__currentRowIndex]
-    
-    def rowSetting(self, row):
-        
-        """
-        行情報を生成する
-        
-        Parameters
-        ----------
-        None
-        
-        """
-        
-        for col in self.__columns:
+        # 編集状態にする
+        self.__editRecrow()
 
-            # Modelのインスタンス変数取得
-            memberInvoke = getattr(self.modelType, col[0])
-            # 既定値の設定
-            setattr(row, col[0], memberInvoke.default.arg)
-        
-        # 生成した行を返す                                 
-        return row
-    
-    def columns(self):
-        
-        """
-        カラム情報プロパティ
-        """
-        
-        return self.__columns
-    
-    def addRow(self, row):
-        
-        """
-        レコードセットに行を追加する
-        
-        Parameters
-        ----------
-        row : Any
-            追加する行情報
-        """
-        
-        self.rows.append(row)
-    
     def delRow(self):
         
         """
         レコードセットのカレント行を削除対象とする
         """
 
-        self.rows[self.__currentRowIndex]._recset__rowState = self.rowState.deleted
-        
+
+        # 行状態をdeletedに変更
+        self.State = self.rowState.deleted
+
     def eof(self):
         
         """
@@ -241,26 +408,9 @@ class recset():
         # カレント行インデックス++
         self.__currentRowIndex += 1
 
-        return False if len(self.rows) > self.__currentRowIndex else True
+        return False if len(self.__rows) > self.__currentRowIndex else True
     
-    def primaryKeys(self):
-        
-        """
-        主キー情報プロパティ
-        """
-        
-        return self.__primaryKeys
-    
-    def getCurrentRow(self):
-        
-        """
-        カレント行を取得する
-        """        
-        
-        # カレント行を返す
-        return self.rows[self.__currentRowIndex]
-    
-    def getDataFrame(self):
+    def getDataFrame(self, __rows = None):
         
         """
         Model⇒DataFrameに変換する
@@ -271,26 +421,26 @@ class recset():
 
         """
         
-        rowlist = []
-        if len(self.rows) == 0:
+        rows = self.__rows if __rows is None else __rows
+        collist = []
+        if len(rows) == 0:
             for column in self.__columns:
-                rowlist.append(column[0])
-        else:            
-            # 行インスタンスをDeepCopy
-            targetRows = copy.deepcopy(self.rows)
-            # DataFrame化で不要な列を削除
-            for rowInstance in targetRows:
-                delattr(rowInstance, '_sa_instance_state')
+                collist.append(column[0])
+        else:
+            
+            # 行情報コピー
+            targetRows = self.__deepCopyRow(rows)
+            
             # 行インスタンスをリスト化
             rowlist = list(map(lambda f: vars(f), targetRows))
 
         # rowlistをDataFrame変換
-        df = pd.DataFrame(rowlist) if len(self.rows) > 0 else pd.DataFrame(columns=rowlist)
+        df = pd.DataFrame(rowlist) if len(rows) > 0 else pd.DataFrame(columns=collist)
         
         # DataFrameに主キー設定
-        if len(self.__primaryKeys) > 0 and len(self.rows) > 0:
+        if len(self.__primaryKeys) > 0 and len(rows) > 0:
             df = df.set_index(self.__primaryKeys, drop=False)
-                
+        
         # 戻り値を返す
         return df
     
@@ -298,6 +448,7 @@ class recset():
         
         """
         レコードセットをフィルタする
+        結果セットはdbから取得
 
         Parameters
         ----------
@@ -305,15 +456,27 @@ class recset():
             where条件
         """
         
+        # 行情報初期化
+        self.__initRows()
+        
+        # フィールドクラス
+        self.__fields = fields(self.__rows, self.__dfrows)
+
         # クエリ実行
-        self.rows = []
+        rowIndex = 0
         q = self.__session.query(self.modelType).filter(w).all()
         for row in q:
             # 行状態をnoChangedに変更
             row.__rowState = self.rowState.noChanged
             # rowsにクエリ結果を追加
-            self.rows.append(row)
-    
+            self.__rows.append(row)
+            # DataFrame - add
+            self.__addRecrowDataFrame(row, rowIndex)
+            rowIndex += 1
+        
+        # カレント行インデックス初期化
+        self.__currentRowIndex = -1
+
     def existsPKeyRec(self, row):
         
         """
@@ -326,10 +489,34 @@ class recset():
         # 主キーを条件として該当レコードが存在するか確認
         w = self.__getPKeyFilter(row)
         q = self.__session.query(self.modelType).filter(w).all()
+        for qrow in q:
+            
+            dictRow = vars(row)
+            dictRow_sorted = sorted(dictRow.items())
+            for col in dictRow_sorted:
+                if col[0] == '_sa_instance_state': continue
+                setattr(qrow, col[0], getattr(row, col[0]))
+            
+            self.__rows.append(qrow)
+            self.rows.remove(row)
 
         # 結果を返す
         return len(q) > 0
 
+    def find(self, w):
+        
+        """
+        現在保持しているレコードセットに指定した条件に合致するレコードが存在するか返す
+
+        w : Any
+            抽出条件
+        """
+        
+        # 条件抽出
+        dfCheck = self.__dfrows[w]
+
+        return True if len(dfCheck) > 0 else False
+
     def upsert(self):
 
         """
@@ -340,7 +527,7 @@ class recset():
             rowState = addedとした行を追加する際に主キー違反している場合、強制的にmodifiedとして扱う。\n
             recsetに存在する追加行(rowState = added)全てに対して存在チェックが走るので \n
             件数によっては更新にかなりの時間を要する。
-            削除行に関してはレコード抽出後にdeleteメソッドを走らせるはずなので存在チェックは行っていない。          
+            削除行に関してはレコード抽出後にdeleteメソッドを走らせるはずなので存在チェックは行っていない。
         """
 
         return self.update(True)
@@ -361,14 +548,14 @@ class recset():
         try:
         
             # 新規行はaddする
-            newRows = [row for row in self.rows if row._recset__rowState == self.rowState.added]
+            newRows = [row for row in self.__rows if row._recset__rowState == self.rowState.added]
             for newRow in newRows:
                 # 主キー違反していない行のみadd
                 if upsert == False or not self.existsPKeyRec(newRow):
                     self.__session.add(newRow)
 
             # 削除行はdeleteする
-            delRows = [row for row in self.rows if row._recset__rowState == self.rowState.deleted]
+            delRows = [row for row in self.__rows if row._recset__rowState == self.rowState.deleted]
             for delRow in delRows:
                 self.__session.delete(delRow)
 
